#include <iostream>
#include <cctype>
#include <SDL.h>

static SDL_Window *window = nullptr;
static SDL_Renderer *renderer = nullptr;

static void exitOnErr(bool predicate, const char *str)
{
  if(predicate)
  {
    std::cout << str << "; " << SDL_GetError() << std::endl;
    if(renderer) SDL_DestroyRenderer(renderer);
    if(window) SDL_DestroyWindow(window);
    SDL_Quit();
    exit(-1);
  }
}

static void printInteresting()
{
  const auto flags = SDL_GetWindowFlags(window);
  std::cout << "SDL_WINDOW_SHOWN=" << (flags & SDL_WINDOW_SHOWN) << std::endl
            << "SDL_WINDOW_INPUT_GRABBED=" << (flags & SDL_WINDOW_INPUT_GRABBED) << std::endl
            << "SDL_WINDOW_INPUT_FOCUS=" << (flags & SDL_WINDOW_INPUT_FOCUS) << std::endl
            << "SDL_WINDOW_MOUSE_FOCUS=" << (flags & SDL_WINDOW_MOUSE_FOCUS) << std::endl
            << "SDL_WINDOW_FOREIGN=" << (flags & SDL_WINDOW_FOREIGN) << std::endl
            << "SDL_WINDOW_MOUSE_CAPTURE=" << (flags & SDL_WINDOW_MOUSE_CAPTURE) << std::endl
            << "SDL_HasScreenKeyboardSupport()=" << (SDL_HasScreenKeyboardSupport() == SDL_TRUE) << std::endl
            << "SDL_IsTextInputActive()=" << SDL_IsTextInputActive() << std::endl
            << "(SDL_GetKeyboardFocus()==window)=" << (SDL_GetKeyboardFocus() == window) << std::endl
            << "SDL_KEYDOWN=" << (SDL_EventState(SDL_KEYDOWN, SDL_QUERY) == SDL_ENABLE) << std::endl
            << "SDL_KEYUP=" << (SDL_EventState(SDL_KEYUP, SDL_QUERY) == SDL_ENABLE) << std::endl
            << "SDL_TEXTEDITING=" << (SDL_EventState(SDL_TEXTEDITING, SDL_QUERY) == SDL_ENABLE) << std::endl
            << "SDL_TEXTINPUT=" << (SDL_EventState(SDL_TEXTINPUT, SDL_QUERY) == SDL_ENABLE) << std::endl
            << "SDL_KEYMAPCHANGED=" << (SDL_EventState(SDL_KEYMAPCHANGED, SDL_QUERY) == SDL_ENABLE) << std::endl
            << "VideoDriver=" << SDL_GetCurrentVideoDriver() << std::endl;
}

int main(int argc, char *argv[])
{
  exitOnErr(SDL_Init(SDL_INIT_EVERYTHING) < 0, "SDL_Init");
  window = SDL_CreateWindow("Keyboard Hell", SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED,
                            640, 480, SDL_WINDOW_SHOWN | SDL_WINDOW_OPENGL);
  exitOnErr(!window, "SDL_CreateWindow");
  renderer = SDL_CreateRenderer(window, -1, SDL_RENDERER_ACCELERATED | SDL_RENDERER_PRESENTVSYNC);
  exitOnErr(!renderer, "SDL_CreateRenderer");
  exitOnErr(SDL_SetRenderDrawColor(renderer, 63, 127, 255, 255) != 0, "SDL_SetRenderDrawColor");

  printInteresting();

  SDL_Event event;
  bool quit = false;
  SDL_Keycode key;
  while(!quit)
  {
    while(SDL_PollEvent(&event))
    {
      switch(event.type)
      {
      case SDL_MOUSEWHEEL:
        std::cout << "mouse_wheel:y=" << event.wheel.y << std::endl;
      break;

      case SDL_MOUSEBUTTONDOWN:
        std::cout << "mouse_button:l=" << (event.button.button == SDL_BUTTON_LEFT)
                  << ",r=" << (event.button.button == SDL_BUTTON_RIGHT)
                  << ",m=" << (event.button.button == SDL_BUTTON_MIDDLE) << std::endl;
      break;

      case SDL_TEXTINPUT:
        std::cout << "text_input:str=" << event.text.text << std::endl;
      break;

      case SDL_TEXTEDITING:
        std::cout << "text_editing:str=" << event.text.text << std::endl;
      break;

      case SDL_KEYDOWN:
      case SDL_KEYUP:
        key = event.key.keysym.sym & ~SDLK_SCANCODE_MASK;
        quit = (key == SDLK_ESCAPE);
        std::cout << "key_" << ((event.type == SDL_KEYDOWN) ? "down" : "up" ) << ":"
                  << "code=" << key << '(' << ((std::isalnum(key)) ? static_cast<char>(key) : '?') << ')'
                  << ",shift=" << ((SDL_GetModState() & KMOD_SHIFT) != 0)
                  << ",ctrl=" << ((SDL_GetModState() & KMOD_CTRL) != 0)
                  << ",alt=" << ((SDL_GetModState() & KMOD_ALT) != 0) << std::endl;
      break;

      case SDL_QUIT:
        quit = true;
      break;
      }

      SDL_RenderClear(renderer);
      SDL_RenderPresent(renderer);
    }
  }

  if(renderer) SDL_DestroyRenderer(renderer);
  if(window) SDL_DestroyWindow(window);
  SDL_Quit();

  return 0;
}
